#!/bin/bash

# Constants
# How long to wait for all CX2/CX3 IOV IB devices to load
MAX_WAIT_FOR_MLX4_IOV_IB_DEV_TO_LOADS=90
# Mellanox PCI vendor ID
PCI_MLX_VEND_ID="15b3"
# CX2 PCI device IDs
PCI_CX2_DEV_IDS="673c|676e"
# CX3 PCI device IDs
PCI_CX3_DEV_IDS="1003"

rdma_script_log_file=/var/log/rdma_script.log
# Print msg to serial console and also save it to a separate log file
rdma_logger()
{
    rdma_msg="RDMA: $1"
    rdma_log_ln="$(date +%c) $(hostname -s) $rdma_msg"
    echo $rdma_msg && echo $rdma_log_ln >> $rdma_script_log_file 2>&1
}

# This function will wait until all CX2/CX3 IOV IB devices are up
# Description:
#If sriov is confured then function waits until it sees
#all mlx4 devices with iov sub directory under /sys/class/infiniband
#num_vfs > 0 (sr_iov in uek2) means that sriov is used.
# Return value:
#0 - Not all devices are up
#1 - All devices are up
wait_for_mlx4_iov_ib_devs_to_come_up()
{
    # Return 0 if not SRIOV mode
    num_vfs=0
    if [ -f /sys/module/mlx4_core/parameters/num_vfs ]; then
num_vfs=$(cat /sys/module/mlx4_core/parameters/num_vfs)
fi
    if [ -f /sys/module/mlx4_core/parameters/sr_iov ]; then
num_vfs=$(cat /sys/module/mlx4_core/parameters/sr_iov)
fi

    if [ -z "$num_vfs" ]; then
return 0
fi

    if [ $num_vfs -eq 0 ]; then
return 0
fi

    # Get number of mlx4 devices
    mlx4_ids="$PCI_CX2_DEV_IDS|$PCI_CX3_DEV_IDS"
    num_of_mlx4_ib_devs=$(lspci -n | awk '{print $3}' |
      grep $PCI_MLX_VEND_ID | grep -E $mlx4_ids|wc -l)

    times=$MAX_WAIT_FOR_MLX4_IOV_IB_DEV_TO_LOADS

    num_of_mlx4_iov_ib_devs_up=0
    rdma_logger "Waiting for $num_of_mlx4_ib_devs CX2/CX3 HCA(s) to come up"
    while [ $times -gt 0 ] &&
          [ $num_of_mlx4_iov_ib_devs_up -lt $num_of_mlx4_ib_devs ];
  do
num_of_mlx4_iov_ib_devs_up=$(ls -d /sys/class/infiniband/mlx4_*/iov | wc -l)
times=$(($times - 1))
sleep 1
done
    
    if [ $num_of_mlx4_iov_ib_devs_up -eq $num_of_mlx4_ib_devs ]; then
rdma_logger "Done"
return 1
else
rdma_logger "Error: Only $num_of_mlx4_iov_ib_devs_up out of
$num_of_mlx4_ib_devs CX2/CX3 HCA(s) found"
return 0
fi
}

do_static_alias_guid()
{
guid_gen_magic=64
byte_to_modify=4

# Skip if SM should assign GUIDs
if [ $(cat /sys/module/mlx4_ib/parameters/sm_guid_assign) -eq 1 ]; then
return
fi

# Skip if uek2 as there we do alias GUID in driver
# guid_gen_magic module parameter exist only in uek2
if [ -f /sys/module/mlx4_ib/parameters/guid_gen_magic ]; then
return
fi

# Wait for all IOV IB devices to come up
wait_for_mlx4_iov_ib_devs_to_come_up
if [ $? -eq 0 ]; then
return
fi

bits_to_shift=$((($byte_to_modify - 1) * 8))

echo -n "Initializing alias GUIDs"
for dev in $(ls /sys/class/infiniband/)
do
echo -n "."
# We do stuff only for mlx4 PF devices
if [ $(echo $dev | grep mlx4 | wc -l) -eq 0 ] ||
   [ ! -d /sys/class/infiniband/$dev/iov/ ]; then
continue;
fi

echo -n "."
for port in $(ls /sys/class/infiniband/$dev/iov/ports/)
do
echo -n "."
idx0_guid=$(cat /sys/class/infiniband/$dev/iov/ports/$port/admin_guids/0)
idx0_guid=$(printf "%x" $((0x$idx0_guid & 0xFFFFFF00FFFFFF)))
for idx in $(ls /sys/class/infiniband/$dev/iov/ports/$port/admin_guids/)
do
if [ $idx -eq 0 ]; then
continue;
fi

delta=$(printf "0x%x" $((0x$guid_gen_magic + $idx)))
delta=$(printf "0x%x" $(($delta << $bits_to_shift)))
guid=$(printf "0x%x\n" $((0x$idx0_guid + $delta)))
echo $guid > /sys/class/infiniband/$dev/iov/ports/$port/admin_guids/$idx
done
done
done
echo "done"
}

do_static_alias_guid

exit 0

